% M-file to create MPC-controllers for use with the quadruple tank process.

clear all
path(path,'../')

% % % Model of the quadruple tank process % % %

% Process parameters
A1=4.9; % [cm^2]
A2=4.9;
A3=4.9;
A4=4.9;

a1=0.03; % [cm^2]
a2=0.03;
a3=0.03;
a4=0.03;

kc=0.5; % [V cm^{-1}]
g=981;  % [cm s^{-2}]

k1=1.6; % [cm^3 V^{-1} s^{-1}]
k2=1.6;

g1=0.3; % ratio of allocated pump capacity between upper
g2=0.3; % and lower tank [1]

% Linearization point
h1 = 10;  % [cm]
h2 = 10;  % [cm]
% h3, h4, v1, v2 from stationarity condition (maple)
v1 = sqrt(2*g)*((g2-1)*a2*sqrt(h2)+g2*a1*sqrt(h1))/(k1*(g1+g2-1)); % [V]
v2 = sqrt(2*g)*((g1-1)*a1*sqrt(h1)+g1*a2*sqrt(h2))/(k2*(g1+g2-1)); % [V]
h3 = (k2*v2*(1-g2)/a3)^2/(2*g); % [cm]
h4 = (k1*v1*(1-g1)/a4)^2/(2*g); % [cm]
 
% Build state space model
T1=A1/a1*sqrt(2*h1/g);
T2=A2/a2*sqrt(2*h2/g);
T3=A3/a3*sqrt(2*h3/g);
T4=A4/a4*sqrt(2*h4/g);

A=[-1/T1 0 A3/(A1*T3) 0;
      0 -1/T2 0 A4/(A2*T4);
      0 0 -1/T3 0;
      0 0 0 -1/T4];

B=[g1*k1/A1 0;
      0 g2*k2/A2;   
      0 (1-g2)*k2/A3;
      (1-g1)*k1/A4 0];

C=[kc 0 0 0; % Notice the measured signals are given in Volts!
      0 kc 0 0];

D=zeros(2,2);

Ts = 0.5; % sample time

% 2nd order LP filters
Tf1 = 0.5; % h1 time constnat[s]
Tf2 = 0.5; % h2 time constant [s]


% Discretize model
[Ad, Bd, Cyd, Dzd]=ssdata(c2d(ss(A,B,C,D),Ts));
Czd = Cyd;
Ccd = kc*eye(4);
Dcd = zeros(4,2);


% % % Controller Design % % %

% Constraints
% limit delta u
du_max = []';
du_min = []';
% limit absolute value of u
u_max = []';
u_min = [];
% limit constrained outputs
z_max = []';
z_min = [];

% Prediction parameters
Hp = 30;  % Prediction horizon
Hu = 10;  % Horizon for varying input signal
Hw = 1;  % First penalty sample
zblk= 2;  % blocking factor 
ublk= 2;  % blocking factor


% Weights
Q = diag([1 1]);
R = diag([1 1]);
W =  diag([1 1 1 1]); %orginal
V =  0.1*diag([1 1]); %orginal


% Create controller
md = MPCInit(Ad,Bd,Cyd,Czd,Dzd,Ccd,Dcd,Hp,Hw,zblk,Hu,ublk, ...
	    du_max,du_min,u_max,u_min,z_max, ...
	    z_min,Q,R,W,V,Ts,2,'qp_as');
